#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

//#include "calcimei.h"

#define OUT_FILE "MP0B_001_NEW"

int main(int argc, char *argv[])
{
    char *str = "357369035621901";


    FILE* output;
    char inp_imei[15];
    char out_imei[12] = {0};
    int i, ret;
    char *mode;

    if (argc < 2 || strlen(argv[1])!=15 || (argc==3 && strlen(argv[2])!=15))
    {
        fprintf(stdout, "Use: %s <IMEI_1 15 digits> [IMEI_2 15 digits]\n", argv[0]);
        return 1;
    }
    for (i=1; i < argc; i++)
    {
        memset(inp_imei, 0, sizeof(inp_imei));
        memset(out_imei, 0, sizeof(out_imei));

        strncpy(inp_imei, argv[i], sizeof(inp_imei));

        fprintf(stdout, "New IMEI %d (15 digits):%s\n", i, inp_imei);

        ret = calc_imei(&inp_imei, &out_imei);
        if ( ret )
        {
            fprintf(stderr, "Invalid IMEI %d format!\n", i);
            return 1;
        }

        mode="w";
        if (i==2)
        {
            mode="a";
        }
        output = fopen(OUT_FILE, mode);
        fwrite(out_imei, 1, sizeof(out_imei), output);
        fclose(output);

    }


    fprintf(stdout, "New IMEI in file = %s\n", OUT_FILE);

    return 0;
}

// Calcula IMEI
int calc_imei(char inp_imei[15], char out_imei[12])
{
    char out_mask[12] = {0xAB, 0xA0, 0x6F, 0x2F, 0x1F, 0x1E, 0x9A, 0x45, 0x0, 0x0, 0x0, 0x0};
    int i=0, j=0;

// 0xAB = 10101011 = 171
// 0xA0 = 10100000 = 160
// 0x6f = 01101111 = 111
// 0x2f = 00101111 = 47
// 0x1f = 00011111 = 31
// 0x1e = 00011110 = 30
// 0x9a = 10011010 = 154
// 0x45 = 01000101 = 45

    for (i=0, j=0; i < 15; i++, j++)
    {
	// Cai fora se não for número
        if (inp_imei[i] < '0' || inp_imei[i] > '9')
        {
            return 1;
        }
	// transforma número ascii em hexa
        out_imei[j] = (inp_imei[i] - '0'); 
	// Termina de analisar se chegou ao fim
        if (i >= 14)
            break;
	// Cai fora se não for número
        if (inp_imei[i+1] < '0' || inp_imei[i+1] > '9')
        {
             return 1; 
        }
	// Coloca:
	//    o 2º algarismo no nibble alto e o 1º algarismo no nibble baixo do 1º byte.
	//    o 4º algarismo no nibble alto e o 3º algarismo no nibble baixo do 2º byte.
	//    o 6º algarismo no nibble alto e o 5º algarismo no nibble baixo do 3º byte, etc.
	// Ou seja, junta dois algarismos num único byte, little endian.
        out_imei[j] += ((inp_imei[i+1] - '0') << 4); 

	// Ex: IMEI 35182904xxxxxxc
	// 0000.0011 = 3
	// 0000.0101 = 5
	// 0101.0000 = 5 << 4 = 80
	// 0101.0011 = 83
	
	// Mascara. Pra disfarçar?
	// 0101.0011 
	// 1010.1011 = 171 XOR  
	// 1111.1000 = 0xF8
        out_imei[j] = out_imei[j] ^ out_mask[j]; // bitwise XOR

	// incrementa índice da string entrada pra pegar de dois em dois algarismos
        i++; 
    }
    // Mascara o último byte (out_imei[7])
    out_imei[j] = out_imei[j] ^ out_mask[j];

    // Constantes usadas no checksum
    // Celular Y803 deve ser 0x55; em vez de 0x57;
    out_imei[8] = 0x55; //Может быть и 0x0, ни на что не влияет // Can be 0x0, or has no effect 
    // Celular Y803 deve ser 0x8B;em vez de 0xDB;
    out_imei[9] = 0x8B; //Может быть и 0x0, ни на что не влияет

    // Calcula checksum?
    out_imei[10] = out_imei[11] = 0;

    for (i = 0; i < 10; i++)
    {
        if (i & 0x1)
        {
            out_imei[11] += out_imei[i]; // sum of odds
        }
        else
        {
            out_imei[10] += out_imei[i]; // sum of evens
        }
    }

    return 0;
}
